public inbox for lvm2-cvs@sourceware.org
help / color / mirror / Atom feed
From: mornfall@sourceware.org
To: lvm-devel@redhat.com, lvm2-cvs@sourceware.org
Subject: LVM2/daemons/lvmetad lvmetad-core.c
Date: Wed, 15 Feb 2012 14:06:00 -0000	[thread overview]
Message-ID: <20120215140649.21656.qmail@sourceware.org> (raw)

CVSROOT:	/cvs/lvm2
Module name:	LVM2
Changes by:	mornfall@sourceware.org	2012-02-15 14:06:48

Modified files:
	daemons/lvmetad: lvmetad-core.c 

Log message:
	In lvmetad, also nuke VGs when all their PVs are stolen by another VG (vgmerge
	& vgsplit do this).

Patches:
http://sourceware.org/cgi-bin/cvsweb.cgi/LVM2/daemons/lvmetad/lvmetad-core.c.diff?cvsroot=lvm2&r1=1.33&r2=1.34

--- LVM2/daemons/lvmetad/lvmetad-core.c	2012/02/15 11:43:06	1.33
+++ LVM2/daemons/lvmetad/lvmetad-core.c	2012/02/15 14:06:48	1.34
@@ -487,26 +487,45 @@
 	return result;
 }
 
+static int vg_remove_if_missing(lvmetad_state *s, const char *vgid);
+
 /* You need to be holding the pvid_to_vgid lock already to call this. */
-static int update_pvid_to_vgid(lvmetad_state *s, struct dm_config_tree *vg, const char *vgid)
+static int update_pvid_to_vgid(lvmetad_state *s, struct dm_config_tree *vg,
+			       const char *vgid, int nuke_empty)
 {
 	struct dm_config_node *pv = pvs(vg->root);
+	struct dm_hash_table *to_check = dm_hash_create(32);
 
 	if (!vgid)
 		return 0;
 
 	while (pv) {
 		const char *pvid = dm_config_find_str(pv->child, "id", NULL);
+		const char *vgid_old = dm_hash_lookup(s->pvid_to_vgid, pvid);
+		if (vgid_old && nuke_empty)
+			dm_hash_insert(to_check, vgid_old, (void*) 1);
 		dm_hash_insert(s->pvid_to_vgid, pvid, (void *) vgid);
 		debug("remap PV %s to VG %s\n", pvid, vgid);
 		pv = pv->sib;
 	}
 
+	struct dm_hash_node *n = dm_hash_get_first(to_check);
+
+	while (n) {
+		const char *check_vgid = dm_hash_get_key(to_check, n);
+		lock_vg(s, check_vgid);
+		vg_remove_if_missing(s, check_vgid);
+		unlock_vg(s, check_vgid);
+		n = dm_hash_get_next(to_check, n);
+	}
+
+	dm_hash_destroy(to_check);
+
 	return 1;
 }
 
-/* A pvid map lock needs to be held. */
-static int remove_metadata(lvmetad_state *s, const char *vgid)
+/* A pvid map lock needs to be held if update_pvids = 1. */
+static int remove_metadata(lvmetad_state *s, const char *vgid, int update_pvids)
 {
 	struct dm_config_tree *old;
 	const char *oldname;
@@ -518,7 +537,8 @@
 	if (!old)
 		return 0;
 
-	update_pvid_to_vgid(s, old, "#orphan");
+	if (update_pvids)
+		update_pvid_to_vgid(s, old, "#orphan", 0);
 	/* need to update what we have since we found a newer version */
 	dm_hash_remove(s->vgid_to_metadata, vgid);
 	dm_hash_remove(s->vgid_to_vgname, vgid);
@@ -554,7 +574,7 @@
 
 	if (missing) {
 		debug("nuking VG %s\n", vgid);
-		remove_metadata(s, vgid);
+		remove_metadata(s, vgid, 0);
 	}
 
 	unlock_pvid_to_status(s);
@@ -629,7 +649,7 @@
 	if (haveseq >= 0 && haveseq < seq) {
 		debug("Updating metadata for %s at %d to %d\n", _vgid, haveseq, seq);
 		/* temporarily orphan all of our PVs */
-		remove_metadata(s, vgid);
+		remove_metadata(s, vgid, 1);
 	}
 
 	// debug_cft("METADATA", metadata);
@@ -640,7 +660,7 @@
 	dm_hash_insert(s->vgname_to_vgid, name, (void *)vgid);
 	unlock_vgid_to_metadata(s);
 
-	update_pvid_to_vgid(s, cft, vgid);
+	update_pvid_to_vgid(s, cft, vgid, 1);
 
 	unlock_pvid_to_vgid(s);
 	retval = 1;
@@ -773,7 +793,7 @@
 	fprintf(stderr, "vg_remove: %s\n", vgid);
 
 	lock_pvid_to_vgid(s);
-	remove_metadata(s, vgid);
+	remove_metadata(s, vgid, 1);
 	unlock_pvid_to_vgid(s);
 
 	return daemon_reply_simple("OK", NULL);


             reply	other threads:[~2012-02-15 14:06 UTC|newest]

Thread overview: 31+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2012-02-15 14:06 mornfall [this message]
  -- strict thread matches above, loose matches on Subject: below --
2012-03-23 10:34 zkabelac
2012-02-27 10:19 zkabelac
2012-02-27 10:10 zkabelac
2012-02-24  0:24 mornfall
2012-02-24  0:12 mornfall
2012-02-21  9:19 mornfall
2012-02-15 17:37 mornfall
2012-02-15 17:30 mornfall
2012-02-15 14:15 mornfall
2012-02-15 11:43 mornfall
2012-02-13 14:25 zkabelac
2012-01-25 21:42 zkabelac
2011-12-18 22:31 mornfall
2011-09-17 13:33 zkabelac
2011-09-02 11:04 zkabelac
2011-07-25 17:59 mornfall
2011-07-25 15:51 mornfall
2011-07-25 15:33 mornfall
2011-07-20 21:33 mornfall
2011-07-20 21:27 mornfall
2011-07-20 21:26 mornfall
2011-07-20 21:23 mornfall
2011-07-20 18:45 mornfall
2011-07-20 18:34 mornfall
2011-07-20 18:24 mornfall
2011-07-20 16:49 mornfall
2011-07-20 16:46 mornfall
2011-07-20 15:14 mornfall
2011-07-19 19:15 mornfall
2011-07-19 14:14 mornfall

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20120215140649.21656.qmail@sourceware.org \
    --to=mornfall@sourceware.org \
    --cc=lvm-devel@redhat.com \
    --cc=lvm2-cvs@sourceware.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).